#include "Mouse.h"
#include "Keyboard.h"

//Definição dos pinos do joystick
const int eixoY = A0;         
const int eixoX = A1;
const int botaoEsquerdoMouse = 2;
const int botaoDireitoMouse = 3;

//Definição dos pinos dos botões que controlam algumas teclas do teclado do PC
const int botaoB = 5;
const int botaoI = 6;
const int botaoT = 7;

//Definição de outros pinos
const int ativaEntrada = 4;
const int pinoLed = 8;

//Configuração controle do joystick
const int sensibilidadeJoystick = 10;
const int delayRespostaMouse = 10;
const int gatilhoInicial = 2; //Faz o mouse não mover sozinho quando o joystick estiver em descanso

//Configuração dos botões de controle do teclado
const int taxaRepeticaoTeclado = 180;

bool entradaAtiva = false;    
int ultimoEstadoAtivaEntrada = HIGH;

unsigned long int controleRespostaMouse = 0;
unsigned long controleTaxaRepeticaoTeclado = 0;

//Obter o valor dos eixos do joystick
int obterEixoJoystick(int eixo)
{
  int valorEixo = analogRead(eixo);
  int valorMinimo = sensibilidadeJoystick - sensibilidadeJoystick*2;
  int valorMaximo = sensibilidadeJoystick;
  
  valorEixo = map(valorEixo, 0, 1023, valorMinimo, valorMaximo);

  if (abs(valorEixo) < gatilhoInicial) 
  {
    valorEixo = 0;
  }

  return valorEixo;
}

//Setar botões direito e esquerdo do mouse
void setBotaoMouse(int pinoBotao)
{
  int botaoMouse;
  if(pinoBotao == botaoEsquerdoMouse)
  {
    botaoMouse = MOUSE_LEFT;
  }
  else
  {
    botaoMouse = MOUSE_RIGHT;
  }

  if(digitalRead(pinoBotao) == LOW) 
  {
    if(!Mouse.isPressed(botaoMouse)) 
    {
      Mouse.press(botaoMouse);
    }
  }
  else 
  {
    if(Mouse.isPressed(botaoMouse)) 
    {
      Mouse.release(botaoMouse);
    }
  }
}

void setup() 
{
  pinMode(botaoEsquerdoMouse, INPUT_PULLUP);
  pinMode(botaoDireitoMouse, INPUT_PULLUP); 

  pinMode(ativaEntrada, INPUT_PULLUP);
  pinMode(pinoLed, OUTPUT);

  pinMode(botaoB, INPUT_PULLUP);
  pinMode(botaoI, INPUT_PULLUP);
  pinMode(botaoT, INPUT_PULLUP);
  
  Mouse.begin();
  Keyboard.begin();
}

void loop() 
{
  int valorEixoX = 0;
  int valorEixoY = 0;
  int estadoLigaEntrada = digitalRead(ativaEntrada);

  //Verifica se o botão de ativação para controle do mouse e teclado foi pressionado
  if (estadoLigaEntrada != ultimoEstadoAtivaEntrada) 
  {
    if (estadoLigaEntrada == LOW) 
    {
      entradaAtiva = !entradaAtiva;
      digitalWrite(pinoLed, entradaAtiva);
    }
  }
  
  ultimoEstadoAtivaEntrada = estadoLigaEntrada;
  
  //Somente controla o mouse e o teclado se o botão de ativação foi pressionado
  if(entradaAtiva)
  {
    if(( millis() - controleRespostaMouse) > delayRespostaMouse)
    {
      //Obtém os valores do joystick e controla o mouse
      valorEixoX = obterEixoJoystick(eixoX);
      valorEixoY = obterEixoJoystick(eixoY);
      Mouse.move(valorEixoX, valorEixoY, 0);
      
      controleRespostaMouse = millis();
    }
    
    //Botões do mouse
    setBotaoMouse(botaoEsquerdoMouse);
    setBotaoMouse(botaoDireitoMouse);
    
    //Controla o teclado
    if((millis() - controleTaxaRepeticaoTeclado) > taxaRepeticaoTeclado)
    {
      if (digitalRead(botaoB) == LOW)
      {
        Keyboard.press('B');
        controleTaxaRepeticaoTeclado = millis();
      }
      
      if (digitalRead(botaoI) == LOW) 
      {
        Keyboard.press('I');
        controleTaxaRepeticaoTeclado = millis();
      }
      
      if (digitalRead(botaoT) == LOW) 
      {
        Keyboard.press('T');
        controleTaxaRepeticaoTeclado = millis();
      }
      
      Keyboard.releaseAll();//Faz com que as teclas não fiquem constantemente pressionadas
    }
  }
  
  delay(delayRespostaMouse);
}
